﻿using System.Linq;
using Elasticsearch.Net;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;

namespace WebApiElasticSearchClient.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class SearchController : ControllerBase
    {
        private readonly ElasticLowLevelClient _lowLevelClient;
        public SearchController(ElasticLowLevelClient lowLevelClient)
        {
            _lowLevelClient = lowLevelClient;
        }

        [HttpGet("Line")]
        public ActionResult Line(string text)
        {
            string queryWithParams = GetLineQuery(text);
            ESResponse eSResponse  = PerformQuery(queryWithParams);

            return Ok(eSResponse.hits.hits.Select(h => new { h.Score, h.PlayLine }));
        }

        [HttpGet("PlayAndLine")]
        public ActionResult PlayAndLine(string play, string text)
        {
            string queryWithParams = GetPlayAndLineQuery(play, text);
            ESResponse eSResponse  = PerformQuery(queryWithParams);

            return Ok(eSResponse.hits.hits.Select(h => new { h.Score, h.PlayLine }));
        }

        // putting this code here for simplicity 
        private string GetLineQuery(string text)
        {
            string elasticSearchQuery = System.IO.File.ReadAllText($"Queries/Line.json");
            dynamic workableElasticSearchQuery = JsonConvert.DeserializeObject(elasticSearchQuery);
            workableElasticSearchQuery.query.match_phrase_prefix.Line = text;

            return workableElasticSearchQuery.ToString();
        }

        private string GetPlayAndLineQuery(string play, string text)
        {
            string elasticSearchQuery = System.IO.File.ReadAllText($"Queries/PlayAndLine.json");
            dynamic workableElasticSearchQuery = JsonConvert.DeserializeObject(elasticSearchQuery);
            workableElasticSearchQuery.query.@bool.must[0].match.Play = play;
            workableElasticSearchQuery.query.@bool.must[1].match_phrase_prefix.Line = text;

            return workableElasticSearchQuery.ToString();
        }

        private ESResponse PerformQuery(string queryWithParams)
        {
            var response = _lowLevelClient.Search<StringResponse>("shakespeare", queryWithParams);
            ESResponse esResponse = System.Text.Json.JsonSerializer.Deserialize<ESResponse>(response.Body);
            return esResponse;
        }
    }
}
